--[[-------------------------------------------------------------------------
-- LitheTooltipDoctor - Put the Game Tooltip wherever you want
--  by The Lithe Development Team 
--  http://www.wowace.com/index.php/Lithe_Development_Team
--  
--  Last Modified: $Date: 2006-12-07 08:43:13 -0500 (Thu, 07 Dec 2006) $
--  Revision: $Revision: 19463 $
--  This software is licensed under a zlib/linpng license. 
--
-- Development Todo/Ideas:
--  *  Please post suggestions at http://www.wowace.com/forums/index.php/topic,1948.0.html
--
-- Bugs
--  * None known
--]]-------------------------------------------------------------------------

-- Local constants
local MAJOR_VERSION = "1.0"
local MINOR_VERSION = tonumber((string.gsub("$Revision: 19463 $", "^.-(%d+).-$", "%1")))

-- Local libraries
local L = AceLibrary("AceLocale-2.2"):new("LitheTooltipDoctor")

-- Options table
local opts = { 
	type='group',
	args = {
		show = {
			type = 'execute',
			name = L["OPT_SHOW_NAME"],
			desc = L["OPT_SHOW_DESC"],
			func = "ShowMoveFrame",
		},
		reset = {
			type = 'execute',
			name = L["OPT_RESET_NAME"],
			desc = L["OPT_RESET_DESC"],
			func = "ResetPosition",
		}
	},
}

-- Initialize the addon
LitheTooltipDoctor = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0", "AceHook-2.1", "AceDebug-2.0")
LitheTooltipDoctor:RegisterChatCommand({L["SLASHCMD_LONG"], L["SLASHCMD_SHORT"]}, opts)
LitheTooltipDoctor:RegisterDB("LitheTooltipDoctorDB", "LitheTooltipDoctorDBPC")
LitheTooltipDoctor.version = MAJOR_VERSION .. "." .. MINOR_VERSION
LitheTooltipDoctor.date = string.gsub("$Date: 2006-12-07 08:43:13 -0500 (Thu, 07 Dec 2006) $", "^.-(%d%d%d%d%-%d%d%-%d%d).-$", "%1")

-- Setup AceDB defaults
LitheTooltipDoctor:RegisterDefaults("profile", {
	point = "CENTER",
	relativePoint = "CENTER",
	xOfs = 0, 
	yOfs = 0,
})

function LitheTooltipDoctor:OnInitialize()
	self.isMoveMode = false
	self:SecureHook("GameTooltip_SetDefaultAnchor")
end

function LitheTooltipDoctor:OnEnable()
	self.tooltipAnchorFrame = self:CreateTooltipAnchorFrame()
	self:RepositionTooltipAnchor()
	self.tooltipAnchorFrame:Hide()
end

function LitheTooltipDoctor:GameTooltip_SetDefaultAnchor(tooltip, parent)
	if self.tooltipAnchorFrame ~= nil then
		-- Borrowed from DFM
		tooltip:SetOwner(parent, "ANCHOR_NONE");
		local prefix = "TOP";
		local suffix = "RIGHT";
		local x, y = UIParent:GetCenter();
		local x2, y2 = self.tooltipAnchorFrame:GetCenter();
		if (x2 < x) then
			suffix = "LEFT";
		end
		if (y2 < y) then
			prefix = "BOTTOM";
		end
		tooltip:SetPoint(prefix..suffix, self.tooltipAnchorFrame, prefix..suffix, 0, 0);
	else
		self.hooks["GameTooltip_SetDefaultAnchor"](tooltip,parent)
	end
end

function LitheTooltipDoctor:ResetPosition()
	self.db.profile.xOfs = 0
	self.db.profile.yOfs = 0
	self.db.profile.point = "CENTER"
	self.db.profile.relativePoint = "CENTER"

	self:RepositionTooltipAnchor()
end

function LitheTooltipDoctor:RepositionTooltipAnchor()
	self.tooltipAnchorFrame:ClearAllPoints()
	self.tooltipAnchorFrame:SetPoint( self.db.profile.point, UIParent, self.db.profile.relativePoint, self.db.profile.xOfs, self.db.profile.yOfs )
end

function LitheTooltipDoctor:ShowMoveFrame()
	if self.DraggableFrame == nil then
		self.draggableFrame = self:CreateTooltipAnchorFrame(true)
	end
	self.draggableFrame:ClearAllPoints()
	self.draggableFrame:SetPoint( self.db.profile.point, UIParent, self.db.profile.relativePoint, self.db.profile.xOfs, self.db.profile.yOfs )
end

function LitheTooltipDoctor:SetNewAnchorLocation()
	local point,relativeTo,relativePoint,xOfs,yOfs = self.draggableFrame:GetPoint()
	self.db.profile.point = point
	self.db.profile.relativePoint = relativePoint
	self.db.profile.xOfs = xOfs
	self.db.profile.yOfs = yOfs

	self.draggableFrame:Hide()
	self:RepositionTooltipAnchor()
end

function LitheTooltipDoctor:CreateTooltipAnchorFrame(movable)
	local frame = CreateFrame("Frame", nil, UIParent)
	frame:SetFrameStrata("DIALOG")
	frame:SetWidth(120)
	frame:SetHeight(80)

	if movable == true then
		frame:EnableMouse(true)
		frame:SetMovable(true)
		frame:RegisterForDrag("LeftButton")
		frame:SetScript("OnDragStart", function() this:StartMoving() end )
		frame:SetScript("OnDragStop", function() 
			this:StopMovingOrSizing() 
		end )
		frame:SetScript("OnMouseDown", function()
			if arg1 == "RightButton" then
				LitheTooltipDoctor:SetNewAnchorLocation()
			end
		end)

		local string = frame:CreateFontString()
		string:SetAllPoints(frame)
		string:SetFontObject("GameFontNormalSmall")
		string:SetText(L["DRAGFRAME_TITLE"].."|n|n"..L["DRAGFRAME_TEXT"])

		frame:SetBackdrop({
				bgFile = "Interface/Tooltips/UI-Tooltip-Background",
				edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
				tile = true,
				tileSize = 16,
				edgeSize = 16,
				insets = { left = 5, right = 5, top = 5, bottom = 5 }
		})
		frame:SetBackdropColor(0.75,0,0,1)
		frame:SetBackdropBorderColor(0.75,0,0,1)
	end

	return frame
end
